"""
Common Copr code for dealing with HTTP requests
"""

import json
import time
from requests import get, post, put, RequestException


class SafeRequest:
    """
    Build a HTTP request and _safely_ send it.

    If the server cannot be reached, the request is repeated either indefinitely
    or until a timeout is reached.
    """

    # Prolong the sleep time before asking frontend again
    SLEEP_INCREMENT_TIME = 5

    # Reasonable timeout for requests that block the client
    TIMEOUT = 2*60

    def __init__(self, auth=None, log=None, try_indefinitely=False):
        self.auth = auth
        self.log = log
        self.try_indefinitely = try_indefinitely

    def get(self, url, **kwargs):
        """
        Issue relentless GET request to a given URL
        """
        return self.send(url, method='get', **kwargs)

    def post(self, url, data, **kwargs):
        """
        Issue relentless POST request to given URL
        """
        return self.send(url, method='post', data=data, **kwargs)

    def put(self, url, data, **kwargs):
        """
        Issue relentless POST request to a given URL
        """
        return self.send(url, method='put', data=data, **kwargs)

    def send(self, url, method, data=None, **kwargs):
        """
        Issue relentless request to a given URL
        """
        return self._send_request_repeatedly(url, method=method, data=data,
                                             **kwargs)

    def _send_request(self, url, method, data=None, **kwargs):
        headers = {"content-type": "application/json"}
        auth = ("user", self.auth) if self.auth else None

        try:
            req_args = {}
            req_args.update(kwargs)
            req_args["auth"] = auth
            req_args["headers"] = headers
            req_args["timeout"] = self.TIMEOUT / 5
            method = method.lower()
            if method in ['post', 'put']:
                req_args['data'] = json.dumps(data)
                method = post if method == 'post' else put
            else:
                method = get
            response = method(url, **req_args)
        except RequestException as ex:
            raise RequestRetryError(
                "Requests error on {}: {}".format(url, str(ex)))

        if response.status_code >= 500:
            # Server error.  Hopefully this is only temporary problem, we wan't
            # to re-try, and wait till the server works again.
            raise RequestRetryError(
                "Request server error on {}: {} {}".format(
                    url, response.status_code, response.reason))

        if response.status_code >= 400:
            # Client error.  The mistake is on our side, it doesn't make sense
            # to continue with retries.
            raise RequestError(
                "Request client error on {}: {} {}".format(
                    url, response.status_code, response.reason))

        # TODO: Success, but tighten the redirects etc.
        return response

    def _send_request_repeatedly(self, url, method, data=None, **kwargs):
        """
        Repeat the request until it succeeds, or timeout is reached.
        """
        sleep = self.SLEEP_INCREMENT_TIME
        start = time.time()
        stop = start + self.TIMEOUT

        i = 0
        while True:
            i += 1
            if not self.try_indefinitely and time.time() > stop:
                raise RequestError(
                    "Attempt to talk to server timeouted "
                    "(we gave it {} attempts)".format(i))

            try:
                return self._send_request(url, method=method, data=data,
                                          **kwargs)
            except RequestRetryError as ex:
                self.log.warning("Retry request #%s on %s: %s", i, url,
                                 str(ex))
                time.sleep(sleep)
                sleep += self.SLEEP_INCREMENT_TIME


class RequestRetryError(Exception):
    """
    Request to server failed, try again
    """


class RequestError(Exception):
    """
    Request to server failed
    """



"""
Common Copr code for working with directories and their contents
"""

import os


def walk_limited(path, maxdepth=None, mindepth=None):
    """
    The same as os.walk(), except that we can control the returned values to
    minimal and maximal depth of traversed directories.  The maximum depth is
    also lowering I/O because we don't actually have to traverse whole tree of
    unused files.
    """
    for dirpath, dirnames, files in os.walk(path):
        raw_subpath = os.path.relpath(dirpath, path)
        subpath = os.path.normpath(raw_subpath)
        depth = 0
        if subpath != ".":
            depth = len(subpath.split(os.sep))
        old_dirnames = dirnames.copy()
        if maxdepth is not None and depth >= maxdepth:
            # Per help(os.walk), we don't want to go deeper:
            # ...
            # When topdown is true, the caller can modify the dirnames list
            # in-place (e.g., via del or slice assignment), and walk will only
            # recurse into the subdirectories whose names remain in dirnames;
            # ...
            del dirnames[:]
        if mindepth is None or depth >= mindepth:
            yield dirpath, old_dirnames, files



# TODO: is there something like python-rpm-utils or python-dnf-utils for this?
def splitFilename(filename):
    """
    Pass in a standard style rpm fullname

    Return a name, version, release, epoch, arch, e.g.::
        foo-1.0-1.i386.rpm returns foo, 1.0, 1, i386
        1:bar-9-123a.ia64.rpm returns bar, 9, 123a, 1, ia64
    """

    if filename[-4:] == '.rpm':
        filename = filename[:-4]

    archIndex = filename.rfind('.')
    arch = filename[archIndex+1:]

    relIndex = filename[:archIndex].rfind('-')
    rel = filename[relIndex+1:archIndex]

    verIndex = filename[:relIndex].rfind('-')
    ver = filename[verIndex+1:relIndex]

    epochIndex = filename.find(':')
    if epochIndex == -1:
        epoch = ''
    else:
        epoch = filename[:epochIndex]

    name = filename[epochIndex + 1:verIndex]
    return name, ver, rel, epoch, arch



import random
import string

from six import with_metaclass

# We don't know how to define the enums without `class`.
# pylint: disable=too-few-public-methods

class EnumType(type):
    def _wrap(cls, attr=None):
        if attr is None:
            raise NotImplementedError
        if isinstance(attr, int):
            for k, v in cls.vals.items():
                if v == attr:
                    return k
            raise KeyError("num {0} is not mapped".format(attr))
        return cls.vals[attr]
    def __call__(cls, attr):
        return cls._wrap(attr)
    def __getattr__(cls, attr):
        return cls._wrap(attr)


class ActionTypeEnum(with_metaclass(EnumType, object)):
    vals = {
        "delete": 0,
        "rename": 1,
        "legal-flag": 2,
        "createrepo": 3,
        "update_comps": 4,
        "gen_gpg_key": 5,
        "rawhide_to_release": 6,
        "fork": 7,
        "update_module_md": 8,
        "build_module": 9,
        "cancel_build": 10,
        "remove_dirs": 11,
    }


class ActionResult(with_metaclass(EnumType, object)):
    vals = {
        'WAITING': 0,
        'SUCCESS': 1,
        'FAILURE': 2,
    }


class DefaultActionPriorityEnum(with_metaclass(EnumType, object)):
    """
    The higher the 'priority' is, the later the task is taken.
    Keep actions priority in range -100 to 100
    """
    vals = {
        "gen_gpg_key": -70,
        "cancel_build": -10,
        "createrepo": 0,
        "fork": 0,
        "build_module": 0,
        "update_comps": 0,
        "delete": 60,
        "rawhide_to_release": 70,
    }


class ActionPriorityEnum(with_metaclass(EnumType, object)):
    """
    Naming/assigning the values is a little bit tricky because
    how the current implementation works (i.e. it is inverted).
    However, from the most abstract point of view,
    "highest priority" means "do this as soon as possible"
    """
    vals = {"highest": -99, "lowest": 99}


class BackendResultEnum(with_metaclass(EnumType, object)):
    vals = {"waiting": 0, "success": 1, "failure": 2}


class RoleEnum(with_metaclass(EnumType, object)):
    vals = {"user": 0, "admin": 1}


class StatusEnum(with_metaclass(EnumType, object)):
    vals = {
        "failed": 0,     # build failed
        "succeeded": 1,  # build succeeded
        "canceled": 2,   # build was canceled
        "running": 3,    # SRPM or RPM build is running
        "pending": 4,    # build(-chroot) is waiting to be picked
        "skipped": 5,    # if there was this package built already
        "starting": 6,   # build was picked by worker but no VM initialized yet
        "importing": 7,  # SRPM is being imported into dist-git
        "forked": 8,     # build(-chroot) was forked
        "waiting": 9,    # build(-chroot) is waiting for something else to finish
        "unknown": 1000, # undefined
    }


def _filtered_status_enum(keys):
    new_values = {}
    for key, value in StatusEnum.vals.items():
        if key in keys:
            new_values[key] = value
    return new_values


class ModuleStatusEnum(StatusEnum):
    vals = _filtered_status_enum(["canceled", "running", "starting", "pending",
                                  "failed", "succeeded", "waiting", "unknown"])


class BuildSourceEnum(with_metaclass(EnumType, object)):
    vals = {"unset": 0,
            "link": 1,  # url
            "upload": 2,  # pkg, tmp, url
            "pypi": 5, # package_name, version, python_versions
            "rubygems": 6, # gem_name
            "scm": 8, # type, clone_url, committish, subdirectory, spec, srpm_build_method
            "custom": 9, # user-provided script to build sources
            "distgit": 10, # distgit_instance, package_name, committish
           }


class FailTypeEnum(with_metaclass(EnumType, object)):
    vals = {"unset": 0,
            # General errors mixed with errors for SRPM URL/upload:
            "unknown_error": 1,
            "build_error": 2,
            "srpm_import_failed": 3,
            "srpm_download_failed": 4,
            "srpm_query_failed": 5,
            "import_timeout_exceeded": 6,
            "git_clone_failed": 31,
            "git_wrong_directory": 32,
            "git_checkout_error": 33,
            "srpm_build_error": 34,
           }



#!/usr/bin/python

import re
import os
import codecs
from setuptools import setup

long_description = """COPR is lightweight build system. It allows you to create new project
in WebUI, and submit new builds and COPR will create yum repository from latest builds.

This package contains python code used by other Copr packages. Mostly
useful for developers only."""


__description__ = "Common python code used by Copr."
__author__ = "Dominik Turecek"
__author_email__ = "dturecek@redhat.com"
__url__ = "https://pagure.io/copr/copr"


setup(
    name='copr-common',
    version="0.14",
    description=__description__,
    long_description=long_description,
    author=__author__,
    author_email=__author_email__,
    url=__url__,
    license='GPLv2+',
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
    ],
    packages=['copr_common'],
    include_package_data=True,
    zip_safe=False
    )
